unit bearlibterminal;

interface

	uses
		ctypes, strings;
		
	type
		color_t = cuint32;
		
	const
		VK_CLOSE			= $03; // Same code as VK_CANCEL from winuser.h
		VK_BACK				= $08; // Backspace
		VK_RETURN			= $0D; // Enter
		VK_SHIFT			= $10;
		VK_CONTROL			= $11;
		VK_PAUSE			= $13; // Pause/Break
		VK_ESCAPE			= $1B;
		VK_SPACE			= $20;
		VK_PRIOR			= $21; // Page Up
		VK_NEXT				= $22; // Page Down
		VK_END				= $23;
		VK_HOME				= $24;
		VK_LEFT				= $25; // Left arrow
		VK_UP				= $26; // Up arrow
		VK_RIGHT			= $27; // Right arrow
		VK_DOWN				= $28; // Down arrow
		VK_INSERT			= $2D;
		VK_DELETE			= $2E;
		VK_NUMPAD0			= $60;
		VK_NUMPAD1			= $61;
		VK_NUMPAD2			= $62;
		VK_NUMPAD3			= $63;
		VK_NUMPAD4			= $64;
		VK_NUMPAD5			= $65;
		VK_NUMPAD6			= $66;
		VK_NUMPAD7			= $67;
		VK_NUMPAD8			= $68;
		VK_NUMPAD9			= $69;
		VK_MULTIPLY			= $6A; // '*' on numpad
		VK_ADD				= $6B; // '+' on numpad
		VK_SUBTRACT			= $6D; // '-' on numpad
		VK_DECIMAL			= $6E; // '.' on numpad
		VK_DIVIDE			= $6F; // '/' on numpad
		VK_F1				= $70;
		VK_F2				= $71;
		VK_F3				= $72;
		VK_F4				= $73;
		VK_F5				= $74;
		VK_F6				= $75;
		VK_F7				= $76;
		VK_F8				= $77;
		VK_F9				= $78;
		VK_F10				= $79;
		VK_F11				= $7A;
		VK_F12				= $7B;
		VK_FLAG_RELEASED	= $100; // This flag is set if key was released instead of pressed
		
	function terminal_open(options: pchar): integer;
		cdecl; external 'bearlibterminal.dll' name 'terminal_open';
		
	procedure terminal_close();
		cdecl; external 'bearlibterminal.dll' name 'terminal_close';
		
	procedure terminal_size(width: integer; height: integer);
		cdecl; external 'bearlibterminal.dll' name 'terminal_size';
		
	procedure terminal_title__dll(title: pchar);
		cdecl; external 'bearlibterminal.dll' name 'terminal_title';
		
	procedure terminal_title(title: string);
		
	function terminal_font__dll(description: pchar): integer;
		cdecl; external 'bearlibterminal.dll' name 'terminal_font';
		
	function terminal_font(description: string): integer;
		
	procedure terminal_begin();
		cdecl; external 'bearlibterminal.dll' name 'terminal_begin';
		
	procedure terminal_end();
		cdecl; external 'bearlibterminal.dll' name 'terminal_end';
		
	procedure terminal_clear();
		cdecl; external 'bearlibterminal.dll' name 'terminal_clear';
		
	procedure terminal_color(color: color_t);
		cdecl; external 'bearlibterminal.dll' name 'terminal_color';
		
	procedure terminal_put(x: integer; y: integer; code: integer);
		cdecl; external 'bearlibterminal.dll' name 'terminal_put';
		
	procedure terminal_puts__dll(x: integer; y: integer; s: pchar);
		cdecl; external 'bearlibterminal.dll' name 'terminal_puts';
		
	procedure terminal_puts(x: integer; y: integer; s: string);
	
	function terminal_kbhit(): integer;
		cdecl; external 'bearlibterminal.dll' name 'terminal_kbhit';
	
	function terminal_get(): integer;
		cdecl; external 'bearlibterminal.dll' name 'terminal_get';
		
	function color_from_argb(a: integer; r: integer; g: integer; b: integer): color_t;
		cdecl; external 'bearlibterminal.dll' name 'color_from_argb';
		
	function color_from_name__dll(name: pchar): color_t;
		cdecl; external 'bearlibterminal.dll' name 'color_from_name';
		
	function color_from_name(name: string): color_t;
		
implementation
	procedure terminal_title(title: string);
	var
		buffer: pchar;
	begin
		buffer := stralloc(Length(title)+1);
		strpcopy(buffer, title);
		terminal_title__dll(buffer);
		strdispose(buffer);
	end;

	function terminal_font(description: string): integer;
	var
		buffer: pchar;
	begin
		buffer := stralloc(Length(description)+1);
		strpcopy(buffer, description);
		terminal_font := terminal_font__dll(buffer);
		strdispose(buffer);
	end;
	
	procedure terminal_puts(x: integer; y: integer; s: string);
	var
		buffer: pchar;
	begin
		buffer := stralloc(Length(s)+1);
		strpcopy(buffer, s);
		terminal_puts__dll(x, y, buffer);
		strdispose(buffer);
	end;
	
	function color_from_name(name: string): color_t;
	var
		buffer: pchar;
	begin
		buffer := stralloc(Length(name)+1);
		strpcopy(buffer, name);
		color_from_name := color_from_name__dll(buffer);
		strdispose(buffer);
	end;
	
end.
